program polygoned_cube;
uses ObjGfx0,ObjGfx8,keys;
const
  divd:word=128;
  dist=150;
  nofpolys=9; { number of polygons -1 }
  nofpoints=11; { number of points -1 }
  point:array[0..nofpoints,0..2] of integer=(
    (-30,-30, 40),( 30,-30, 40),( 50,-50,  0),( 30,-30,-40),
    (-30,-30,-40),(-50,-50,  0),(-30, 30, 40),( 30, 30, 40),
    ( 50, 50,  0),( 30, 30,-40),(-30, 30,-40),(-50, 50,  0));
  planes:array[0..nofpolys,0..3] of byte=(
    (0,1,7,6),(1,2,8,7),(9,8,2,3),(10,9,3,4),(10,4,5,11),
    (6,11,5,0),(0,1,2,5),(5,2,3,4),(6,7,8,11),(11,8,9,10));
type polytype=array[0..nofpolys] of integer;

type Tcube=object(TGfx8)
      stab:array[0..319] of integer;
      polyz,pind:polytype;
      constructor init;
      procedure run;
      function cosinus(i:word):integer;
      function sinus(i:word):integer;
      procedure quicksort(lo,hi:integer);
      destructor done; virtual;
     end;
var
  screen:PScreen8;
  cube:^Tcube;

constructor Tcube.init;
var i:word;
begin
 inherited init;
 getmem(410,240);
 new(screen,init);
 screen^.setupmode(_640x480x8bpp);
 for i:=1 to 255 do screen^.setrgbpalette(i,i div 16,i div 8,i div 4);
 for i:=0 to 319 do stab[i]:=round(sin(i*pi/128)*divd);
end;

{ -------------------------------------------------------------------------- }

function Tcube.cosinus(i:word):integer;
begin
 cosinus:=stab[i+64]
end;

function Tcube.sinus(i:word):integer;
begin
 sinus:=stab[i]
end;

{ -------------------------------------------------------------------------- }

procedure Tcube.quicksort(lo,hi:integer);

procedure sort(l,r:integer);
var i,j,x,y:integer;
begin
  i:=l; j:=r; x:=polyz[(l+r) div 2];
  repeat
    while polyz[i]<x do inc(i);
    while x<polyz[j] do dec(j);
    if i<=j then begin
      y:=polyz[i]; polyz[i]:=polyz[j]; polyz[j]:=y;
      y:=pind[i]; pind[i]:=pind[j]; pind[j]:=y;
      inc(i); dec(j);
    end;
  until i>j;
  if l<j then sort(l,j);
  if i<r then sort(i,r);
end;

begin
  sort(lo,hi);
end;

{ -------------------------------------------------------------------------- }

procedure tcube.run;
const xst=2; yst=1 ; zst=-1;
var
  xp,yp,z:array[0..nofpoints] of integer;
  polypoints:array[0..nofpoints] of PointType;
  x,y,i,j,k:integer;
  n,phix,phiy,phiz:byte;
  count:word;
begin
  phix:=0; phiy:=0; phiz:=0;
  SetkbdInterrupt;
  repeat
   clear(0);
    for n:=0 to nofpoints do begin
      i:=(cosinus(phiy)*point[n,0]-sinus(phiy)*point[n,2]) div divd;
      j:=(cosinus(phiz)*point[n,1]-sinus(phiz)*i) div divd;
      k:=(cosinus(phiy)*point[n,2]+sinus(phiy)*point[n,0]) div divd;
      x:=(cosinus(phiz)*i+sinus(phiz)*point[n,1]) div divd;
      y:=(cosinus(phix)*j+sinus(phix)*k) div divd;
      z[n]:=(cosinus(phix)*k-sinus(phix)*j) div divd;
      xp[n]:=(GetMaxX div 2)+cosinus(phix)+(-x*dist) div (z[n]-dist);
      yp[n]:=(GetMaxY div 2)-sinus(phiy) div 2+(-y*dist) div (z[n]-dist);
    end;
    for n:=0 to nofpolys do begin
      polyz[n]:=z[planes[n,0]]+z[planes[n,1]]+z[planes[n,2]]+z[planes[n,3]];
      pind[n]:=n;
    end;
    quicksort(0,nofpolys);
    for n:=5 to nofpolys  do
     begin
      for count:=0 to 3 do
       begin
        polypoints[count].x:=xp[planes[pind[n],count]];
        polypoints[count].y:=yp[planes[pind[n],count]]
       end;
      fillconvexpoly(4,polypoints,polyz[n]+64)
     end;
    inc(phix,xst);
    inc(phiy,yst);
    inc(phiz,zst);
    screen^.waitforretrace;
    if (screen^.getmaxx=getmaxx) and (screen^.getmaxy=getmaxy) then
     screen^.flip(self)
    else
     screen^.copybuf((screen^.getmaxx-getmaxx) div 2,(screen^.getmaxy-getmaxy) div 2,
                      self,0,0,getmaxx,getmaxy);
  until keypressed;
  restorekbdinterrupt;
end;

destructor Tcube.done;
begin
 inherited done;
 dispose(screen,done)
end;

begin
 new(cube,init);
 cube^.run;
 dispose(cube,done)
end.
