uses keys,objgfx0,objgfx8;

const SPEED               = 3;
const PLUS                = 1;
const MINUS               = 0;
const MAX_COLORS          = 256;
const MAX_COLOR_PALETTES  = 8;

const MAXX                = 320;
const MAXY                = 200;
const VIDMODE             = _320x200x8bpp;

type Tpoint = record
      x, y, x1, y1:integer;
      dirx, diry:integer;
      dirx1, diry1:integer;
     end;

type TNewGfx8Buf = object(TGfx8)
      public
       procedure averagepix;
     end;
type PLinefire = ^TLinefire;
     TLinefire = object(TScreen8)
      public
       constructor init;
       procedure change_pals(pal_num:integer);
       procedure set_colors;
       procedure run;
       destructor done; virtual;
      private
       buf                : TNewGfx8Buf;
       palettes:array[0..MAX_COLOR_PALETTES-1,0..MAX_COLORS-1] of PLTEChunkEntry;
       chkflg             : byte;
       point              : TPoint;
     end;

{$L linefire} {linefire.asm}
{$F+}
procedure TNewGfx8Buf.averagepix; external;
{$F-}
constructor TLinefire.init;
begin
 inherited init;
 randomize;
 buf.init;
 if not buf.getmem(MAXX,MAXY+1) then
  begin
   writeln('Error allocating memory for double buffer');
   halt
  end;

 if not setmode(VIDMODE) then
  begin
   writeln('Error setting video mode');
   halt
  end;

 point.x := (random(32000) mod 158)+1;
 point.y := (random(32000) mod 98)+1;
 point.x1 := (random(32000) mod 158)+1;
 point.y1 := (random(32000) mod 98)+1;
 point.dirx:=PLUS;
 point.diry:=MINUS;
 point.dirx1:=MINUS;
 point.diry1:=PLUS;
 set_colors;
 makepal;
end;

procedure TLinefire.change_pals(pal_num:integer);
var ctr:integer;
begin
 for ctr:=0 to MAX_COLORS-1 do
  begin
    if(pal[ctr].r < palettes[pal_num][ctr].r) then
     with pal[ctr] do
     begin
       inc(r);
       setrgbpalette(ctr,r,g,b);
     end else
    if(pal[ctr].r > palettes[pal_num][ctr].r) then
     with pal[ctr] do
     begin

      dec(r);
      setrgbpalette(ctr,r,g,b);
     end;

    if(pal[ctr].g < palettes[pal_num][ctr].g) then
     with pal[ctr] do
     begin
       inc(g);
       setrgbpalette(ctr,r,g,b);
     end else
    if(pal[ctr].g > palettes[pal_num][ctr].g) then
     with pal[ctr] do
     begin
       dec(g);
       setrgbpalette(ctr,r,g,b);
     end;

    if(pal[ctr].b < palettes[pal_num][ctr].b) then
     with pal[ctr] do
     begin
       inc(b);
       setrgbpalette(ctr,r,g,b);
     end else
    if(pal[ctr].b > palettes[pal_num][ctr].b) then
     with pal[ctr] do
     begin
      dec(b);
      setrgbpalette(ctr,r,g,b);
     end;

    if((pal[ctr].r = palettes[pal_num][ctr].r) and
           (pal[ctr].g = palettes[pal_num][ctr].g) and
               (pal[ctr].b = palettes[pal_num][ctr].b)) then
      chkflg:= 0
    else
      chkflg:= 1;
  end;
end;

procedure TLinefire.set_colors;
var index,ctr:integer;
begin

  index:=0;
  ctr:=1;
  for ctr:=0 to 63 do
   begin
    pal[index].r := ctr;               {this one is red}
    pal[index+1].r := ctr;
    pal[index+2].r := ctr;
    pal[index+3].r := ctr;
    pal[index].g := 0;
    pal[index+1].g := 0;
    pal[index+2].g := 0;
    pal[index+3].g := 0;
    pal[index].b := 0;
    pal[index+1].b := 0;
    pal[index+2].b := 0;
    pal[index+3].b := 0;
    inc(index,4);
   end;

  index:=0;
  for ctr:=0 to 63 do
   begin
    palettes[0][index].r := 0;
    palettes[0][index+1].r := 0;
    palettes[0][index+2].r := 0;
    palettes[0][index+3].r := 0;
    palettes[0][index].g := ctr;         {this one is green}
    palettes[0][index+1].g := ctr;
    palettes[0][index+2].g := ctr;
    palettes[0][index+3].g := ctr;
    palettes[0][index].b := 0;
    palettes[0][index+1].b := 0;
    palettes[0][index+2].b := 0;
    palettes[0][index+3].b := 0;
    inc(index,4);
   end;

  index:=0;
  for ctr:=0 to 63 do
  begin
    palettes[1][index].r := 0;
    palettes[1][index+1].r := 0;
    palettes[1][index+2].r := 0;
    palettes[1][index+3].r := 0;
    palettes[1][index].g := 0;
    palettes[1][index+1].g := 0;
    palettes[1][index+2].g := 0;
    palettes[1][index+3].g := 0;
    palettes[1][index].b := ctr;
    palettes[1][index+1].b := ctr;        {this one is blue}
    palettes[1][index+2].b := ctr;
    palettes[1][index+3].b := ctr;
    inc(index,4);
  end;

  index:=0;
  for ctr:=0 to 63 do
   begin
    palettes[2][index].r := ctr;
    palettes[2][index+1].r := ctr;
    palettes[2][index+2].r := ctr;
    palettes[2][index+3].r := ctr;
    palettes[2][index].g := ctr;
    palettes[2][index+1].g := ctr;           {white}
    palettes[2][index+2].g := ctr;
    palettes[2][index+3].g := ctr;
    palettes[2][index].b := ctr;
    palettes[2][index+1].b := ctr;
    palettes[2][index+2].b := ctr;
    palettes[2][index+3].b := ctr;
    inc(index,4);
   end;

   index:=0;
   for ctr:=0 to 63 do
    begin
    palettes[3][index].r := ctr;                {red+green}
    palettes[3][index+1].r := ctr;
    palettes[3][index+2].r := ctr;
    palettes[3][index+3].r := ctr;
    palettes[3][index].g := ctr;
    palettes[3][index+1].g := ctr;
    palettes[3][index+2].g := ctr;
    palettes[3][index+3].g := ctr;
    palettes[3][index].b := 0;
    palettes[3][index+1].b := 0;
    palettes[3][index+2].b := 0;
    palettes[3][index+3].b := 0;
    inc(index,4);
   end;

   index:=0;
   for ctr:=0 to 63 do
    begin
     palettes[4][index].r := ctr;           {red+blue}
     palettes[4][index+1].r := ctr;
     palettes[4][index+2].r := ctr;
     palettes[4][index+3].r := ctr;
     palettes[4][index].g := 0;
     palettes[4][index+1].g := 0;
     palettes[4][index+2].g := 0;
     palettes[4][index+3].g := 0;
     palettes[4][index].b := ctr;
     palettes[4][index+1].b := ctr;
     palettes[4][index+2].b := ctr;
     palettes[4][index+3].b := ctr;
     inc(index,4);
    end;

   index:=0;
   for ctr:=0 to 63 do
    begin
    palettes[5][index].r := 0;
    palettes[5][index+1].r := 0;
    palettes[5][index+2].r := 0;
    palettes[5][index+3].r := 0;
    palettes[5][index].g := ctr;
    palettes[5][index+1].g := ctr;
    palettes[5][index+2].g := ctr;
    palettes[5][index+3].g := ctr;
    palettes[5][index].b := ctr;
    palettes[5][index+1].b := ctr;             {//green+blue}
    palettes[5][index+2].b := ctr;
    palettes[5][index+3].b := ctr;
     inc(index,4);
    end;
end;

procedure Tlinefire.run;
var v,xx,yy,temp,temp1,temp2,temp3:integer;
    randd1,randd2,randd3,randd4,randd5,randd6,randd7,randd8:integer;
begin
 randd1 := (random(32000)mod 4)+1;
 randd2 := (random(32000)mod 1)+1;
 randd3 := (random(32000)mod 2)+1;
 randd4 := (random(32000)mod 3)+1;
 randd5 := (random(32000)mod 3)+1;
 randd6 := (random(32000)mod 4)+1;
 randd7 := (random(32000)mod 2)+1;
 randd8 := (random(32000)mod 3)+1;
 chkflg:=1;
 v:=0;
 repeat
  if chkflg=0 then
    v:=random(10) mod 6;
 change_pals(v);

 if(point.x > buf.getmaxx-10) then
 begin
   point.dirx := MINUS;
   randd1 := (random(32000) mod SPEED)+1;
 end;
 if(point.x < 10) then
 begin
   point.dirx := PLUS;
   randd2 := (random(32000) mod SPEED)+1;
 end;
 if(point.y > buf.getmaxy-10) then
 begin
   point.diry := MINUS;
   randd3 := (random(32000) mod SPEED)+1;
 end;
 if(point.y < 10) then
 begin
   point.diry := PLUS;
   randd4 := (random(32000) mod SPEED)+1;
 end;

 if(point.x1 > buf.getmaxx-10) then
 begin
   point.dirx1 := MINUS;
   randd5 := (random(32000) mod SPEED)+1;
 end;
 if(point.x1 < 10) then
 begin
   point.dirx1 := PLUS;
   randd6 := (random(32000) mod SPEED)+1;
 end;
 if(point.y1 > buf.getmaxy-10 ) then
 begin
   point.diry1 := MINUS;
   randd7 := (random(32000) mod SPEED)+1;
 end;
 if(point.y1 < 10) then
 begin
   point.diry1 := PLUS;
   randd8 := (random(32000) mod SPEED)+1;
 end;

{                       //move the points}
 if(point.dirx = PLUS) then inc(point.x,randd1);
 if(point.dirx = MINUS) then dec(point.x,randd2);
 if(point.diry = PLUS) then inc(point.y,randd3);
 if(point.diry = MINUS) then dec(point.y,randd4);

 if(point.dirx1 = PLUS) then inc(point.x1,randd5);
 if(point.dirx1 = MINUS) then dec(point.x1,randd6);
 if(point.diry1 = PLUS) then inc(point.y1,randd7);
 if(point.diry1 = MINUS) then dec(point.y1,randd8);

  with buf, point do
   begin
    line(x,y,x1,y1,245);
    line(getmaxx-x,getmaxy-y,getmaxx-x1,getmaxy-y1,245);
    line(x+1,y+1,x1+1,y1+1,245);
    line(getmaxx-x+1,getmaxy-y+1,getmaxx-x1+1,getmaxy-y1+1,245);
    rectangle(0,0,getmaxx,getmaxy,0)
   end;
  buf.averagepix;
  flip(buf);
 until keypressed;
 while keypressed do readkey;
end;

destructor Tlinefire.done;
begin
 inherited done;
 buf.done;
end;

var LineFire:PLineFire;
begin
 new(Linefire,init);
 Linefire^.run;
 dispose(Linefire,done);
end.