(* ---------------------------- WAV3D.PAS ------------------------------- *)
(* Coded bye Jare/Iguana in 1993. Want more comment's? Write'em!          *)
(* Use this as you like; you're going to anyway, so who cares.            *)
(* But remember: proper crediting and greeting rules.                     *)

{ Rewritten by TCA of NewOrder for high-res}
{ This version uses a separate buffer for mirroring}

USES keys,ObjGfx0,ObjGfx8;

const
  divd:word=128;
  dist=150;
  point:array[0..7,0..2] of integer=(
     (-40,-40,-40),(-40,-40,40),(40,-40,40),(40,-40,-40),
    (-40, 40,-40),(-40, 40,40),(40, 40,40),(40, 40,-40));
  planes:array[0..5,0..3] of byte=(
    (0,4,5,1),(0,3,7,4),(0,1,2,3),(4,5,6,7),(7,6,2,3),(1,2,6,5));

type Tcube=object(Tgfx8)
     phix,phiy,phiz:byte;
      stab:array[0..319] of integer;
      polyz:array[0..5] of integer;
      pind:array[0..5] of byte;
      constructor init;
      procedure runonce;
      function cosinus(i:word):integer;
      function sinus(i:word):integer;
      procedure quicksort(lo,hi:integer);
      destructor done; virtual;
     end;

constructor Tcube.init;
var i:word;
begin
 inherited init;
 getmem(160,150);
 for i:=0 to 319 do stab[i]:=round(sin(i*pi/128)*divd);
 phix:=0; phiy:=0; phiz:=0;

end;

{ -------------------------------------------------------------------------- }

function Tcube.cosinus(i:word):integer;
begin
 cosinus:=stab[i+64]
end;

function Tcube.sinus(i:word):integer;
begin
 sinus:=stab[i]
end;

{ -------------------------------------------------------------------------- }

procedure Tcube.quicksort(lo,hi:integer);

procedure sort(l,r:integer);
var i,j,x,y:integer;
begin
  i:=l; j:=r; x:=polyz[(l+r) div 2];
  repeat
    while polyz[i]<x do inc(i);
    while x<polyz[j] do dec(j);
    if i<=j then begin
      y:=polyz[i]; polyz[i]:=polyz[j]; polyz[j]:=y;
      y:=pind[i]; pind[i]:=pind[j]; pind[j]:=y;
      inc(i); dec(j);
    end;
  until i>j;
  if l<j then sort(l,j);
  if i<r then sort(i,r);
end;

begin
 sort(lo,hi)
end;

{ -------------------------------------------------------------------------- }

procedure tcube.runonce;
const xst=-1; yst=1; zst=-2;
var
  xp,yp,zp:array[0..7] of integer;
  polypoints:array[0..3] of PointType;
  x,y,i,j,k,z:integer;
  n:byte;
  count:word;
begin
   clear(0);
    for n:=0 to 7 do begin
      i:=(cosinus(phiy)*point[n,0]-sinus(phiy)*point[n,2]) div divd;
      j:=(cosinus(phiz)*point[n,1]-sinus(phiz)*i) div divd;
      k:=(cosinus(phiy)*point[n,2]+sinus(phiy)*point[n,0]) div divd;
      x:=(cosinus(phiz)*i+sinus(phiz)*point[n,1]) div divd;
      y:=(cosinus(phix)*j+sinus(phix)*k) div divd;
      z:=(cosinus(phix)*k-sinus(phix)*j) div divd;
      xp[n]:=(GetMaxX div 2)+(-x*dist) div (z-dist);
      yp[n]:=(GetMaxY div 2)+(-y*dist) div (z-dist);
      zp[n]:=z;
    end;
    for n:=0 to 5 do
     begin
      polyz[n]:=(zp[planes[n,0]]+zp[planes[n,1]]+zp[planes[n,2]]+zp[planes[n,3]]) div 4;
      pind[n]:=n
     end;
    quicksort(0,5);
    for n:=3 to 5  do
     begin
      for count:=0 to 3 do
       begin
        polypoints[count].x:=xp[planes[pind[n],count]];
        polypoints[count].y:=yp[planes[pind[n],count]]
       end;
      fillconvexpoly(4,polypoints,polyz[n]+20)
     end;
    inc(phix,xst);
    inc(phiy,yst);
    inc(phiz,zst);
end;

destructor Tcube.done;
begin
 inherited done;
end;

const xsize=279;
      Radius=279.0;
TYPE
   TWavBuf = ARRAY [0..63, 0..127] OF BYTE;
   TLandBuf = ARRAY [0..31, 0..xsize] OF BYTE;

  (* -------------------- *)
type Tripple=object(Tgfx8)
      Buf:Tgfx8;

      Wb :^TWavBuf;
      lb :^TLandBuf;
      highest : ARRAY [0..xsize] OF BYTE;
      k:integer;
      constructor init;
      destructor done; virtual;
      procedure GenerateLand;
      procedure GenerateWav;
      procedure RunOnce;
     end;

CONST
   s31       = 31.0*31.0;
   MaxRadius = Radius*Radius + s31;
   s159       = Radius*Radius;


constructor Tripple.Init;
begin
 inherited init;
 if not GetMem((xsize+1)*2,200) then exit;
 new(wb);
 new(lb);
 fillchar(highest,sizeof(highest),0);
 GenerateLand;
 GenerateWav;
 k:=0;
 buf.init;
 buf.aliasGfx8(self,(xsize+1),0,getmaxx,getmaxy);
end;

destructor Tripple.Done;
begin
 inherited done;
 buf.done;
 dispose(wb);
 dispose(lb)
end;

procedure Tripple.GenerateLand;
var i,j:integer;
begin
 for i := 0 to 31 do
  for j := 0 to xsize do
   begin
    lb^[i,j] := ROUND(SQRT((i+0.0)*i/s31 + (j+0.0)*j/s159)*127{/SQRT(2)});
    if lb^[i,j] > 127 then lb^[i,j] := 127
   end
END;

procedure Tripple.GenerateWav;
var r, d, v, a : REAL;
    i,j:integer;
begin
 for j := 0 to 63 do
  for i := 0 to 127 do
   begin
    r := (i+1) * 5.0 * PI / 320.0;
    d := j * PI / 32.0;
    v := ((Sin(r - d)-Sin(-d)) / r ) - (Cos(r-d)/2);
    a := Cos(i*PI/256);
    Wb^[j, i] := BYTE(100 + ROUND(100.0 * a * a * v))
   end
end;

procedure Tripple.RunOnce;
VAR i, j, n : INTEGER;
    c, lasth, limit : WORD;
begin
 n:=(k + 1) MOD 64;
 FOR j := 0 TO xsize DO
  begin
   c:=63+64;
   lasth := getmaxy;
   for i := 31 downto 0 DO
    begin
     limit := 56+i+Wb^[n,Lb^[i,j]];
     WHILE lasth > limit DO
      begin
       putpixel(xsize-j,lasth,c);
       DEC(lasth)
      end;
     DEC(c)
    end;
   for i := 0 to 31 do
    begin
    limit := 55-i+Wb^[n,Lb^[i,j]];
     WHILE lasth > limit DO
      begin
       putpixel(xsize-j,lasth,c);
       DEC(lasth)
      end;
     DEC(c)
    end;
   if lasth>highest[j] then
    vertline(xsize-j,lasth,highest[j],0);
   highest[j]:=lasth
  end;
 k:=n;
 buf.flip(self);
 buf.flipy;
end;
var screen:PScreen8;
    ripple:Tripple;
    cube:Tcube;
    w,v:word;
BEGIN
 writeln('Calculating...');
 ripple.Init;
 cube.init;
 new(Screen,init);
 if not screen^.setmode(_800x600x8bpp) then
  begin
   writeln('Mode 800x600x8bpp (103h) is not available');
   halt
  end;
 for w:=63 to 127 do
  screen^.setrgbpalette(w,0,0,w);
 for w:=1 to 63 do screen^.setrgbpalette(w,w,0,63-(w div 2));
 ripple.RunOnce;
 for v:=0 to ripple.GetMaxY do
  with screen^ do
   begin
    line(0,v,(GetMaxX-ripple.GetMaxX) shr 1,v,ripple.getpixel(0,ripple.GetMaxY-v));
    line(GetMaxX,v,
         ((GetMaxX-ripple.GetMaxX) shr 1)+ripple.GetMaxX,
         v,ripple.getpixel(0,ripple.GetMaxY-v));
    line(0,((GetMaxY-ripple.GetMaxY))+v,
         (GetMaxX-ripple.GetMaxX) shr 1,
         ((GetMaxY-ripple.GetMaxY))+v,ripple.getpixel(0,v));
    line(GetMaxX,((GetMaxY-ripple.GetMaxY))+v,
         ((GetMaxX-ripple.GetMaxX) shr 1)+ripple.GetMaxX,
         ((GetMaxY-ripple.GetMaxY))+v,ripple.getpixel(0,v));
   end;
 repeat
  ripple.RunOnce;
  cube.runonce;
  screen^.copybuf((screen^.GetMaxX-ripple.GetMaxX) shr 1,
                 (Screen^.GetMaxY-ripple.GetMaxY+60),
                 ripple,0,60,ripple.GetMaxX,ripple.GetMaxY);

  screen^.copybuf((screen^.getmaxx-cube.getmaxx) div 2,(screen^.getmaxy-cube.getmaxy) div 2,
                  cube,0,0,cube.getmaxx,cube.getmaxy);
  ripple.FlipX;
  cube.runonce;

  screen^.copybuf((screen^.GetMaxX-ripple.GetMaxX) shr 1,0,
                 ripple,0,0,ripple.GetMaxX,ripple.GetMaxY-60);

  screen^.copybuf((screen^.getmaxx-cube.getmaxx) div 2,(screen^.getmaxy-cube.getmaxy) div 2,
                  cube,0,0,cube.getmaxx,cube.getmaxy);

  ripple.FlipX;

 until keypressed;
 cube.done;
 ripple.Done;
 dispose(screen,done);
END.
