program VesaInformation;
uses objgfx8,Font8,keys;
const FontFile='ROM8X16.DPF';
type ScreenObject=object(TScreen8)
      Font:TDPFont8;
      CurMode:word;
      constructor init;
      procedure run;
      procedure DrawInfo;
      procedure ModeSet(ModeNum:word);
      procedure abort(const s:string);
     end;

function HexW(W : Word) : string;
const
  Digits : array[0..$F] of Char = '0123456789ABCDEF';
    {-Return hex string for word}
begin
 HexW[0] := #4;
 HexW[1] := Digits[hi(W) shr 4];
 HexW[2] := Digits[hi(W) and $F];
 HexW[3] := Digits[lo(W) shr 4];
 HexW[4] := Digits[lo(W) and $F];
end;

function IntToStr(I: Longint): String;
{ Convert any integer type to a string }
var
 S: string[11];
begin
 Str(I, S);
 IntToStr := S;
end;

constructor ScreenObject.init;
begin
 inherited init;
 Font.Init(self);
 if not Font.Load(FontFile) then
  abort('Error loading font file');
 if IsVesaInstalled<>VESA_ok then abort('No VESA BIOS Extension found');
 if not IsModeAvailable(_640x480x8bpp) then abort('Unable to set 640x480x8bpp');
 ModeSet(_640x480x8bpp);

end;

procedure ScreenObject.ModeSet(ModeNum:word);
begin
 if CurMode=ModeNum then exit;
 if not SetMode(ModeNum) then exit;
 CurMode:=ModeNum;
end;

procedure ScreenObject.abort(const s:string);
begin
 done;
 writeln('Error occured: ',s);
 halt
end;

procedure ScreenObject.DrawInfo;
var s:string;
    chheight:word;
begin
 with Font do
  begin
   SetColor(15);
   puttext(0,0,'VESA version:');
   puttext(160,0,IntToStr(VesaRec^.MajVersion)+'.'+IntToStr(VesaRec^.MinVersion));
   puttext(0,20,'Screen Mode:');
   puttext(160,20,HexW(CurMode)+'h');
   puttext(0,40,'X-Resolution:');
   puttext(160,40,IntToStr(xRes));
   puttext(0,60,'Y-Resolution:');
   puttext(160,60,IntToStr(yRes));
   puttext(0,80,'Bits per pixel:');
   puttext(160,80,'8');
   puttext(0,100,'Bytes Per Line:');
   puttext(160,100,IntToStr(ModeRec^.BytesPerLine));
   puttext(0,120,'Linear Frame Buffer:');
   if UsingLFB then
    begin
     puttext(160,120,'Available');
     puttext(0,140,'LFB Address:');
     with ModeRec^ do
      puttext(160,140,IntToStr(PhysBasePtr)+' ('+IntToStr(PhysBasePtr div longint(1048576))+'M)');
    end
   else
    begin
     puttext(160,120,'Not Available');
     puttext(0,140,'Granularity:');
     puttext(160,140,IntToStr(ModeRec^.Granularity)+'k');
     puttext(0,160,'Window A Seg:');
     puttext(160,160,HexW(ModeRec^.WindowASeg)+'h');
     puttext(0,180,'Window B Seg:');
     puttext(160,180,HexW(ModeRec^.WindowBSeg)+'h');
    end;
   chheight:=TextHeight('M');
   puttext(0,GetMaxY-chHeight*5,'F6: 640x400x8bpp');
   puttext(0,GetMaxY-chHeight*4,'F7: 640x480x8bpp');
   puttext(0,GetMaxY-chHeight*3,'F8: 800x600x8bpp');
   puttext(0,GetMaxY-chHeight*2,'F9: 1024x768x8bpp');
   puttext(0,GetMaxY-chHeight,'F10: 1280x1024x8bpp');
  end; {with}
end;

procedure ScreenObject.run;
var ky:byte;
    LastMode:word;
begin
 SetKbdInterrupt;
 DrawInfo;
 repeat
  LastMode:=CurMode;
  ky:=readScanCode;
  case ky of
   KY_F6:ModeSet(_640x400x8bpp);
   KY_F7:ModeSet(_640x480x8bpp);
   KY_F8:ModeSet(_800x600x8bpp);
   KY_F9:ModeSet(_1024x768x8bpp);
   KY_F10:ModeSet(_1280x1024x8bpp);
  end;
  if LastMode<>CurMode then DrawInfo;
 until ky in [KY_Q,KY_ESC];
 RestoreKbdInterrupt;
end;

var screen:ScreenObject;
begin
 screen.init;
 screen.run;
 screen.done
end.